@@ -22,13 +22,16 @@
    Author: Ettore Perazzoli <ettore@gnu.org> */
 
 #include <config.h>
+
 #include "gnome-vfs-cancellation-private.h"
-#include "GNOME_VFS_Daemon.h"
 
 #include "gnome-vfs-utils.h"
 #include "gnome-vfs-private-utils.h"
-#include "gnome-vfs-client.h"
-#include "gnome-vfs-client-call.h"
+
+#ifdef USE_DAEMON
+#include <gnome-vfs-dbus-utils.h>
+#endif
+
 #include <unistd.h>
 
 #ifdef G_OS_WIN32
@@ -42,26 +45,30 @@
    be allowed to call `gnome_vfs_cancellation_cancel()'.  *All* the code is
    based on this assumption.  */
 
-
 struct GnomeVFSCancellation {
 	gboolean cancelled;
 	gint pipe_in;
 	gint pipe_out;
-	GnomeVFSClientCall *client_call;
+#ifdef USE_DAEMON
+	/* daemon handle */
+	gint32 handle;
+	gint32 connection;
+#endif
 };
 
 G_LOCK_DEFINE_STATIC (pipes);
-G_LOCK_DEFINE_STATIC (client_call);
+#ifdef USE_DAEMON
+G_LOCK_DEFINE_STATIC (callback);
+#endif
 
-
 /**
  * gnome_vfs_cancellation_new:
  * 
- * Create a new GnomeVFSCancellation object for reporting cancellation to a
- * GNOME VFS module.
+ * Create a new #GnomeVFSCancellation object for reporting cancellation to a
+ * gnome-vfs module.
  * 
  * Return value: A pointer to the new GnomeVFSCancellation object.
- **/
+ */
 GnomeVFSCancellation *
 gnome_vfs_cancellation_new (void)
 {
@@ -71,17 +78,19 @@ gnome_vfs_cancellation_new (void)
 	new->cancelled = FALSE;
 	new->pipe_in = -1;
 	new->pipe_out = -1;
-	new->client_call = NULL;
-
+#ifdef USE_DAEMON
+	new->connection = 0;
+	new->handle = 0;
+#endif	
 	return new;
 }
 
 /**
  * gnome_vfs_cancellation_destroy:
- * @cancellation: A GnomeVFSCancellation object
+ * @cancellation: a #GnomeVFSCancellation object.
  * 
  * Destroy @cancellation.
- **/
+ */
 void
 gnome_vfs_cancellation_destroy (GnomeVFSCancellation *cancellation)
 {
@@ -91,38 +100,43 @@ gnome_vfs_cancellation_destroy (GnomeVFSCancellation *cancellation)
 		close (cancellation->pipe_in);
 		close (cancellation->pipe_out);
 	}
-	/* Can't have outstanding calls when destroying the cancellation */
-	g_assert (cancellation->client_call == NULL);
 	
 	g_free (cancellation);
 }
 
+#ifdef USE_DAEMON
+
 void
-_gnome_vfs_cancellation_add_client_call (GnomeVFSCancellation *cancellation,
-					 GnomeVFSClientCall *client_call)
+_gnome_vfs_cancellation_set_handle (GnomeVFSCancellation *cancellation,
+				    gint32 connection, gint32 handle)
 {
-	G_LOCK (client_call);
+	G_LOCK (callback);
+
 	/* Each client call uses its own context/cancellation */
-	g_assert (cancellation->client_call == NULL);
-	
-	cancellation->client_call = client_call;
-	G_UNLOCK (client_call);
+	g_assert (cancellation->handle == 0);
+
+	cancellation->connection = connection;
+	cancellation->handle = handle;
+
+	G_UNLOCK (callback);
 }
 
 void
-_gnome_vfs_cancellation_remove_client_call (GnomeVFSCancellation *cancellation,
-					    GnomeVFSClientCall *client_call)
+_gnome_vfs_cancellation_unset_handle (GnomeVFSCancellation *cancellation)
 {
-	G_LOCK (client_call);
-	g_assert (cancellation->client_call == client_call);
+	G_LOCK (callback);
 	
-	cancellation->client_call = NULL;
-	G_UNLOCK (client_call);
+	cancellation->connection = 0;
+	cancellation->handle = 0;
+
+	G_UNLOCK (callback);
 }
 
+#endif
+
 /**
  * gnome_vfs_cancellation_cancel:
- * @cancellation: A GnomeVFSCancellation object
+ * @cancellation: a #GnomeVFSCancellation object.
  * 
  * Send a cancellation request through @cancellation.
  *
@@ -130,16 +144,15 @@ _gnome_vfs_cancellation_remove_client_call (GnomeVFSCancellation *cancellation,
  * callbacks, there is a small race condition where the
  * operation finished callback will be called even if you
  * cancelled the operation. Its the apps responsibility
- * to handle this. See gnome_vfs_async_cancel for more
+ * to handle this. See gnome_vfs_async_cancel() for more
  * discussion about this.
- **/
+ */
 void
 gnome_vfs_cancellation_cancel (GnomeVFSCancellation *cancellation)
 {
-	GNOME_VFS_AsyncDaemon daemon;
-	GnomeVFSClient *client;
-	GnomeVFSClientCall *client_call;
-
+#ifdef USE_DAEMON
+	gint32 handle, connection_id;
+#endif	
 	g_return_if_fail (cancellation != NULL);
 
 	if (cancellation->cancelled)
@@ -147,44 +160,63 @@ gnome_vfs_cancellation_cancel (GnomeVFSCancellation *cancellation)
 
 	if (cancellation->pipe_out >= 0)
 		write (cancellation->pipe_out, "c", 1);
+#ifdef USE_DAEMON
+	handle = 0;
+	connection_id = 0;
 
-	client_call = CORBA_OBJECT_NIL;
-	G_LOCK (client_call);
-	if (cancellation->client_call != NULL) {
-		/* We need to delay the finishing of the client call to avoid
-		 * the cancel call below to cancel the next call in the job thread
-		 * if the job finishes after we drop the lock.
-		 */
-		_gnome_vfs_client_call_delay_finish (cancellation->client_call);
-		client_call = cancellation->client_call;
-		bonobo_object_ref (client_call);
+	G_LOCK (callback);
+	if (cancellation->handle) {
+		handle = cancellation->handle;
+		connection_id = cancellation->connection;
 	}
-	G_UNLOCK (client_call);
-
+	G_UNLOCK (callback);
+#endif	
 	cancellation->cancelled = TRUE;
 
-	if (client_call != NULL) {
-		CORBA_Environment ev;
-		client = _gnome_vfs_get_client ();
-		daemon = _gnome_vfs_client_get_async_daemon (client);
+#ifdef USE_DAEMON
+	if (handle != 0) {
+		DBusConnection *conn;
+		DBusMessage *message;
+		DBusError error;
 
-		CORBA_exception_init (&ev);
-		GNOME_VFS_AsyncDaemon_Cancel (daemon, BONOBO_OBJREF (client_call), &ev);
-		CORBA_exception_free (&ev);
-		_gnome_vfs_client_call_delay_finish_done (client_call);
-		bonobo_object_unref (client_call);
-		CORBA_Object_release (daemon, NULL);
+		dbus_error_init (&error);
+        
+		conn = _gnome_vfs_get_main_dbus_connection ();
+
+		if (conn != NULL) {
+					
+			message = dbus_message_new_method_call (DVD_DAEMON_SERVICE,
+								DVD_DAEMON_OBJECT,
+								DVD_DAEMON_INTERFACE,
+								DVD_DAEMON_METHOD_CANCEL);
+			dbus_message_set_auto_start (message, TRUE);
+			if (!message) {
+				g_error ("Out of memory");
+			}
+
+			if (!dbus_message_append_args (message,
+						       DBUS_TYPE_INT32, &handle,
+						       DBUS_TYPE_INT32, &connection_id,
+						       DBUS_TYPE_INVALID)) {
+				g_error ("Out of memory");
+			}
+
+			dbus_connection_send (conn, message, NULL);
+			dbus_connection_flush (conn);
+			dbus_message_unref (message);
+		}
 	}
+#endif
 }
 
 /**
  * gnome_vfs_cancellation_check:
- * @cancellation: A GnomeVFSCancellation object
+ * @cancellation: a #GnomeVFSCancellation object.
  * 
  * Check for pending cancellation.
  * 
  * Return value: %TRUE if the operation should be interrupted.
- **/
+ */
 gboolean
 gnome_vfs_cancellation_check (GnomeVFSCancellation *cancellation)
 {
@@ -196,13 +228,13 @@ gnome_vfs_cancellation_check (GnomeVFSCancellation *cancellation)
 
 /**
  * gnome_vfs_cancellation_ack:
- * @cancellation: A GnomeVFSCancellation object
+ * @cancellation: a #GnomeVFSCancellation object.
  * 
  * Acknowledge a cancellation.  This should be called if
- * `gnome_vfs_cancellation_check()' returns %TRUE or if `select()' reports that
+ * gnome_vfs_cancellation_check() returns %TRUE or if select() reports that
  * input is available on the file descriptor returned by
- * `gnome_vfs_cancellation_get_fd()'.
- **/
+ * gnome_vfs_cancellation_get_fd().
+ */
 void
 gnome_vfs_cancellation_ack (GnomeVFSCancellation *cancellation)
 {
@@ -225,21 +257,21 @@ gnome_vfs_cancellation_ack (GnomeVFSCancellation *cancellation)
 
 /**
  * gnome_vfs_cancellation_get_fd:
- * @cancellation: A GnomeVFSCancellation object
+ * @cancellation: a #GnomeVFSCancellation object.
  * 
  * Get a file descriptor -based notificator for @cancellation.  When
  * @cancellation receives a cancellation request, a character will be made
  * available on the returned file descriptor for input.
  *
  * This is very useful for detecting cancellation during I/O operations: you
- * can use the `select()' call to check for available input/output on the file
+ * can use the select() call to check for available input/output on the file
  * you are reading/writing, and on the notificator's file descriptor at the
  * same time.  If a data is available on the notificator's file descriptor, you
  * know you have to cancel the read/write operation.
  * 
  * Return value: the notificator's file descriptor, or -1 if starved of
  *               file descriptors.
- **/
+ */
 gint
 gnome_vfs_cancellation_get_fd (GnomeVFSCancellation *cancellation)
 {
