@@ -32,7 +32,10 @@
 #include <gmodule.h>
 #include <libgnomevfs/gnome-vfs-module.h>
 #include <libgnomevfs/gnome-vfs-transform.h>
+#include <libgnomevfs/gnome-vfs-method.h>
+#ifdef USE_DAEMON
 #include "gnome-vfs-daemon-method.h"
+#endif
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
@@ -47,7 +50,7 @@ struct _ModuleElement {
 	const char *args;
 	GnomeVFSMethod *method;
 	GnomeVFSTransform *transform;
-	int nusers;
+	GnomeVFSMethodShutdownFunc shutdown_function;
 	gboolean run_in_daemon;
 };
 typedef struct _ModuleElement ModuleElement;
@@ -93,10 +96,24 @@ _gnome_vfs_get_daemon_force_probe_callback (void)
 	return daemon_force_probe_callback;
 }
 
+static void
+module_element_free (gpointer elementp)
+{
+	ModuleElement *element = (ModuleElement *) elementp;
+	if (element->shutdown_function != NULL) {
+		(* element->shutdown_function) (element->method);
+	}
+
+	/* NB: this is the key */
+	g_free (element->name);
+	g_free (element);
+}
+
 static gboolean
 init_hash_table (void)
 {
-	module_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	module_hash = g_hash_table_new_full (g_str_hash, g_str_equal,
+					     NULL, module_element_free);
 
 	return TRUE;
 }
@@ -192,7 +209,8 @@ gnome_vfs_method_init (void)
 
 static void
 load_module (const gchar *module_name, const char *method_name, const char *args,
-	     GnomeVFSMethod **method, GnomeVFSTransform **transform)
+	     GnomeVFSMethod **method, GnomeVFSTransform **transform,
+	     GnomeVFSMethodShutdownFunc *shutdown_function)
 {
 	GModule *module;
 	GnomeVFSMethod *temp_method = NULL;
@@ -200,12 +218,12 @@ load_module (const gchar *module_name, const char *method_name, const char *args
 	
 	GnomeVFSMethodInitFunc init_function = NULL;
 	GnomeVFSTransformInitFunc transform_function = NULL;
-	GnomeVFSMethodShutdownFunc shutdown_function = NULL;
 
 	*method = NULL;
 	*transform = NULL;
+	*shutdown_function = NULL;
 
-	module = g_module_open (module_name, G_MODULE_BIND_LAZY);
+	module = g_module_open (module_name, G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL);
 	if (module == NULL) {
 		g_warning ("Cannot load module `%s' (%s)", module_name, g_module_error ());
 		return;
@@ -216,11 +234,11 @@ load_module (const gchar *module_name, const char *method_name, const char *args
 	g_module_symbol (module, GNOME_VFS_MODULE_TRANSFORM,
 			 (gpointer *) &transform_function);
 	g_module_symbol (module, GNOME_VFS_MODULE_SHUTDOWN,
-			 (gpointer *) &shutdown_function);
+			 (gpointer *) shutdown_function);
 	
-	if ((init_function == NULL || shutdown_function == NULL) &&
+	if ((init_function == NULL || *shutdown_function == NULL) &&
 	    (transform_function == NULL)) {
-		g_warning ("module '%s' has no init function; may be an out-of-date module", module_name);
+		g_warning ("module '%s' does not have init, transform and shutfown functions; may be an out-of-date module", module_name);
 		return;
 	}
 
@@ -285,7 +303,8 @@ load_module (const gchar *module_name, const char *method_name, const char *args
 
 static void
 load_module_in_path_list (const gchar *base_name, const char *method_name, const char *args,
-			  GnomeVFSMethod **method, GnomeVFSTransform **transform)
+			  GnomeVFSMethod **method, GnomeVFSTransform **transform,
+			  GnomeVFSMethodShutdownFunc *shutdown_function)
 {
 	GList *p;
 
@@ -299,7 +318,7 @@ load_module_in_path_list (const gchar *base_name, const char *method_name, const
 		path = p->data;
 		name = g_module_build_path (path, base_name);
 
-		load_module (name, method_name, args, method, transform);
+		load_module (name, method_name, args, method, transform, shutdown_function);
 		g_free (name);
 
 		if (*method != NULL || *transform != NULL)
@@ -312,6 +331,7 @@ gnome_vfs_add_module_to_hash_table (const gchar *name)
 {
 	GnomeVFSMethod *method = NULL;
 	GnomeVFSTransform *transform = NULL;
+	GnomeVFSMethodShutdownFunc shutdown_function = NULL;
 	ModuleElement *module_element;
 	const char *module_name;
 #if defined (HAVE_SETEUID) || defined (HAVE_SETRESUID)
@@ -356,9 +376,9 @@ gnome_vfs_add_module_to_hash_table (const gchar *name)
 #endif
 		
 		if (g_path_is_absolute (module_name))
-			load_module (module_name, name, args, &method, &transform);
+			load_module (module_name, name, args, &method, &transform, &shutdown_function);
 		else
-			load_module_in_path_list (module_name, name, args, &method, &transform);
+			load_module_in_path_list (module_name, name, args, &method, &transform, &shutdown_function);
 		
 #if defined(HAVE_SETEUID)
 		seteuid (saved_uid);
@@ -371,7 +391,9 @@ gnome_vfs_add_module_to_hash_table (const gchar *name)
 		setresgid (-1, saved_gid, -1);
 #endif
 	} else {
+#ifdef USE_DAEMON
 		method = _gnome_vfs_daemon_method_get ();
+#endif
 	}
 
 	if (method == NULL && transform == NULL)
@@ -380,6 +402,7 @@ gnome_vfs_add_module_to_hash_table (const gchar *name)
 	module_element->name = g_strdup (name);
 	module_element->method = method;
 	module_element->transform = transform;
+	module_element->shutdown_function = shutdown_function;
 	module_element->run_in_daemon = run_in_daemon;
 
 	g_hash_table_insert (module_hash, module_element->name, module_element);
@@ -427,3 +450,18 @@ gnome_vfs_transform_get (const gchar *name)
 	module_element = gnome_vfs_add_module_to_hash_table (name);
 	return module_element ? module_element->transform : NULL;
 }
+
+void
+_gnome_vfs_method_shutdown (void)
+{
+	G_LOCK (gnome_vfs_method_init);
+
+	if (module_hash != NULL) {
+		g_hash_table_destroy (module_hash);
+		module_hash = NULL;
+	}
+
+	method_already_initialized = FALSE;
+
+	G_UNLOCK (gnome_vfs_method_init);
+}
