@@ -23,7 +23,7 @@
 
 #include <config.h>
 
-#include <glib/gmain.h>
+#include <glib.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 #include <libgnomevfs/gnome-vfs-init.h>
 #include <stdio.h>
